@@ -13,24 +13,35 @@ from rich.logging import RichHandler
 
 FORMAT = "%(message)s"
 logging.basicConfig(
-    level=logging.INFO, format=FORMAT, datefmt="[%X]",
-    handlers=[RichHandler(console=Console(stderr=True))]
+    level=logging.INFO,
+    format=FORMAT,
+    datefmt="[%X]",
+    handlers=[RichHandler(console=Console(stderr=True))],
 )
 log = logging.getLogger("rich")
 log.setLevel(logging.INFO)
 
 sys.path.append(
-    os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))))
-#os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')
+    os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
+)
+# os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'settings')
 
 import django
 
 django.setup()
 
-from preservationdatabase.models import CarinianaPreservation, \
-    ClockssPreservation, HathiPreservation, LockssPreservation, \
-    OculScholarsPortalPreservation, PKPPreservation, PorticoPreservation, \
-    LastFill
+from preservationdatabase.models import (
+    CarinianaPreservation,
+    ClockssPreservation,
+    HathiPreservation,
+    LockssPreservation,
+    OculScholarsPortalPreservation,
+    PKPPreservation,
+    PorticoPreservation,
+    LastFill,
+    ISSN,
+    ISSNL,
+)
 
 from django.db import transaction
 import utils
@@ -42,10 +53,12 @@ def cli():
 
 
 @click.command()
-@click.option('--url',
-              default='https://api.portico.org/kbart/Portico_Holding_KBart.txt',
-              help='The URL to fetch')
-@click.option('--local', is_flag=True, default=False)
+@click.option(
+    "--url",
+    default="https://api.portico.org/kbart/Portico_Holding_KBart.txt",
+    help="The URL to fetch",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
 def import_portico(url, local):
     """Download and import data from Portico"""
@@ -53,10 +66,12 @@ def import_portico(url, local):
 
 
 @click.command()
-@click.option('--url',
-              default='https://reports.clockss.org/keepers/keepers-CLOCKSS-report.csv',
-              help='The URL to fetch')
-@click.option('--local', is_flag=True, default=False)
+@click.option(
+    "--url",
+    default="https://reports.clockss.org/keepers/keepers-CLOCKSS-report.csv",
+    help="The URL to fetch",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
 def import_clockss(url, local):
     """Download and import data from CLOCKSS"""
@@ -64,10 +79,12 @@ def import_clockss(url, local):
 
 
 @click.command()
-@click.option('--url',
-              default='https://reports.lockss.org/keepers/keepers-LOCKSS-report.csv',
-              help='The URL to fetch')
-@click.option('--local', is_flag=True, default=False)
+@click.option(
+    "--url",
+    default="https://reports.lockss.org/keepers/keepers-LOCKSS-report.csv",
+    help="The URL to fetch",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
 def import_lockss(url, local):
     """Download and import data from LOCKSS"""
@@ -75,10 +92,12 @@ def import_lockss(url, local):
 
 
 @click.command()
-@click.option('--url',
-              default='https://pkp.sfu.ca/files/pkppn/onix.csv',
-              help='The URL to fetch')
-@click.option('--local', is_flag=True, default=False)
+@click.option(
+    "--url",
+    default="https://pkp.sfu.ca/files/pkppn/onix.csv",
+    help="The URL to fetch",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
 def import_pkp(url, local):
     """Download and import data from PKP's private LOCKSS network"""
@@ -86,10 +105,12 @@ def import_pkp(url, local):
 
 
 @click.command()
-@click.option('--url',
-              default='http://reports-lockss.ibict.br/keepers/pln/ibictpln/keepers-IBICTPLN-report.csv',
-              help='The URL to fetch')
-@click.option('--local', is_flag=True, default=False)
+@click.option(
+    "--url",
+    default="http://reports-lockss.ibict.br/keepers/pln/ibictpln/keepers-IBICTPLN-report.csv",
+    help="The URL to fetch",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
 def import_cariniana(url, local):
     """Download and import data from Cariniana"""
@@ -97,34 +118,45 @@ def import_cariniana(url, local):
 
 
 @click.command()
-@click.option('--file',
-              default='hathi_full_20230101.txt',
-              help='The filename of the Hathitrust full dump to use')
-@click.option('--bucket',
-              default='preservation.research.crossref.org',
-              help='The s3 bucket from which to retrieve the data')
+@click.option(
+    "--file",
+    default="hathi_full_20230101.txt",
+    help="The filename of the Hathitrust full dump to use",
+)
+@click.option(
+    "--bucket",
+    default="preservation.research.crossref.org",
+    help="The s3 bucket from which to retrieve the data",
+)
+@click.option("--local", is_flag=True, default=False)
 @transaction.atomic()
-def import_hathi(file, bucket):
+def import_hathi(file, bucket, local):
     """Import data from Hathi (requires local file download or S3)"""
-    s3client = boto3.client('s3')
+    s3client = boto3.client("s3")
     HathiPreservation.import_data(
-        file, bucket=bucket, s3client=s3client)
+        file, bucket=bucket, s3client=s3client, local=local
+    )
 
 
 @click.command()
-@click.option('--file',
-              default='scholars_portal_keepers_20230202.xml',
-              help='The filename of the OCUL full dump to use')
-@click.option('--bucket',
-              default='preservation.research.crossref.org',
-              help='The s3 bucket from which to retrieve the data')
+@click.option(
+    "--file",
+    default="scholars_portal_keepers_20230202.xml",
+    help="The filename of the OCUL full dump to use",
+)
+@click.option(
+    "--bucket",
+    default="preservation.research.crossref.org",
+    help="The s3 bucket from which to retrieve the data",
+)
 @transaction.atomic()
 def import_ocul(file, bucket):
     """Import data from Ocul (requires local file download or S3)"""
 
-    s3client = boto3.client('s3')
-    OculScholarsPortalPreservation.import_data(file, bucket=bucket,
-                                               s3client=s3client)
+    s3client = boto3.client("s3")
+    OculScholarsPortalPreservation.import_data(
+        file, bucket=bucket, s3client=s3client
+    )
 
     return
 
@@ -146,6 +178,14 @@ def clear_cache():
     LastFill.clear()
 
 
+@click.command()
+@click.argument("issnl_file")
+@transaction.atomic()
+def import_issnl(issnl_file):
+    """Import ISSN-L mappings"""
+    utils.import_issnl(issnl_file, ISSNL, ISSN)
+
+
 @click.command()
 @transaction.atomic()
 def show_cache():
@@ -154,16 +194,21 @@ def show_cache():
     for lf in LastFill.objects.all():
         logging.info(lf)
         time_delta = datetime.datetime.now(pytz.utc) - lf.last_fill_date
-        logging.info('Last cache stamp for {} was {}'.format(
-            lf.archive_name,
-            humanfriendly.format_timespan(
-                humanfriendly.coerce_seconds(time_delta))))
+        logging.info(
+            "Last cache stamp for {} was {}".format(
+                lf.archive_name,
+                humanfriendly.format_timespan(
+                    humanfriendly.coerce_seconds(time_delta)
+                ),
+            )
+        )
 
         if lf.cache_valid:
-            logging.info('{} will use cached version'.format(lf.archive_name))
+            logging.info("{} will use cached version".format(lf.archive_name))
         else:
-            logging.info('{} will be fetched from source'.format(
-                lf.archive_name))
+            logging.info(
+                "{} will be fetched from source".format(lf.archive_name)
+            )
 
 
 @click.command()
@@ -177,7 +222,7 @@ def show_archives():
 
 
 @click.command()
-@click.argument('issn')
+@click.argument("issn")
 @transaction.atomic()
 def show_issn(issn):
     """Show preservation items that match an ISSN"""
@@ -187,20 +232,18 @@ def show_issn(issn):
         issns = archive_object.objects.filter(issn=issn)
 
         for issn_object in issns:
-            logging.info(f'{archive_object.name()} preserves {issn_object}')
+            logging.info(f"{archive_object.name()} preserves {issn_object}")
 
-        if 'eissn' in archive_object._meta.get_fields():
+        if "eissn" in archive_object._meta.get_fields():
             issns = archive_object.objects.filter(eissn=issn)
 
             for issn_object in issns:
-                logging.info(f'{archive_object.name()} preserves {issn_object}')
+                logging.info(f"{archive_object.name()} preserves {issn_object}")
 
 
 @click.command()
-@click.argument('archive_name')
-@click.option('--count',
-              default=25,
-              help='Give random samples for an archive')
+@click.argument("archive_name")
+@click.option("--count", default=25, help="Give random samples for an archive")
 @transaction.atomic()
 def random_samples(archive_name, count):
     """Return random samples that occur in and out of an archive"""
@@ -214,7 +257,7 @@ def random_samples(archive_name, count):
             archive = archive_object
 
     if not archive:
-        log.error(f'Archive {archive_name} not found')
+        log.error(f"Archive {archive_name} not found")
         return
 
     random_entries = utils.random_db_entries(archive, count)
@@ -237,10 +280,10 @@ def random_samples(archive_name, count):
                 break
 
     for entry in random_entries:
-        logging.info('{} contains: {}'.format(archive_name, entry))
+        logging.info("{} contains: {}".format(archive_name, entry))
 
     for entry in not_in_archive:
-        logging.info('{} does not contain: {}'.format(archive_name, entry))
+        logging.info("{} does not contain: {}".format(archive_name, entry))
 
 
 @click.command()
@@ -248,29 +291,34 @@ def import_all():
     """Download and import all data (excluding HathiTrust)"""
 
     import_clockss(
-        url='https://reports.clockss.org/keepers/keepers-CLOCKSS-report.csv'
+        url="https://reports.clockss.org/keepers/keepers-CLOCKSS-report.csv"
     )
 
     import_portico(
-        url='https://api.portico.org/kbart/Portico_Holding_KBart.txt'
+        url="https://api.portico.org/kbart/Portico_Holding_KBart.txt"
     )
 
     import_lockss(
-        url='https://reports.lockss.org/keepers/keepers-LOCKSS-report.csv'
+        url="https://reports.lockss.org/keepers/keepers-LOCKSS-report.csv"
     )
 
     import_cariniana(
-        url='http://reports-lockss.ibict.br/keepers/pln/ibictpln/keepers-IBICTPLN-report.csv'
+        url="http://reports-lockss.ibict.br/keepers/pln/ibictpln/keepers-IBICTPLN-report.csv"
     )
 
 
 @click.command()
-@click.argument('doi')
-def show_preservation(doi):
+@click.argument("doi")
+@click.option(
+    "--literal", help="Use literal DOI (no schema)", is_flag=True, default=False
+)
+def show_preservation(doi, literal=False):
     """
     Determine whether a DOI is preserved
     """
-    doi = utils.normalize_doi(doi)
+    if not literal:
+        doi = utils.normalize_doi(doi)
+
     preservation_statuses, doi = utils.show_preservation_for_doi(doi)
 
     for key, value in preservation_statuses.items():
@@ -278,23 +326,25 @@ def show_preservation(doi):
 
         if preserved:
             if done:
-                log.info(f'[green]Preserved:[/] in {key}',
-                         extra={'markup': True})
+                log.info(
+                    f"[green]Preserved:[/] in {key}", extra={"markup": True}
+                )
             else:
-                log.info(f'[yellow]Preserved (in progress):[/] '
-                         f'in {key}',
-                         extra={'markup': True})
+                log.info(
+                    f"[yellow]Preserved (in progress):[/] " f"in {key}",
+                    extra={"markup": True},
+                )
         else:
-            log.info(f'[red]Not preserved:[/] in {key}',
-                     extra={'markup': True})
+            log.info(f"[red]Not preserved:[/] in {key}", extra={"markup": True})
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     cli.add_command(clear_cache)
     cli.add_command(import_all)
     cli.add_command(import_cariniana)
     cli.add_command(import_clockss)
     cli.add_command(import_hathi)
+    cli.add_command(import_issnl)
     cli.add_command(import_lockss)
     cli.add_command(import_ocul)
     cli.add_command(import_pkp)
