@@ -37,8 +37,9 @@ class LastFill(models.Model):
     @staticmethod
     def get_last_fill(archive_name):
         try:
-            return \
-                LastFill.objects.get(archive_name=archive_name).last_fill_date
+            return LastFill.objects.get(
+                archive_name=archive_name
+            ).last_fill_date
         except LastFill.DoesNotExist:
             return None
 
@@ -56,7 +57,7 @@ class LastFill(models.Model):
             return False
 
     def __str__(self):
-        return '{}: {}'.format(self.archive_name, self.last_fill_date)
+        return "{}: {}".format(self.archive_name, self.last_fill_date)
 
 
 class Publisher(models.Model):
@@ -75,9 +76,9 @@ class LockssPreservation(models.Model):
         db_table = "preservationData_locksspreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['eissn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["eissn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -87,18 +88,23 @@ class LockssPreservation(models.Model):
     preserved_years = models.TextField()
     in_progress_volumes = models.TextField()
     in_progress_years = models.TextField()
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (eissn: {}) (v{}({})) ' \
-               '(in progress: {} ({})) '.format(self.title, self.issn,
-                                                self.eissn,
-                                                self.preserved_volumes,
-                                                self.preserved_years,
-                                                self.in_progress_volumes,
-                                                self.in_progress_years
-                                                )
+        return (
+            "{} (issn: {}) (eissn: {}) (v{}({})) "
+            "(in progress: {} ({})) ".format(
+                self.title,
+                self.issn,
+                self.eissn,
+                self.preserved_volumes,
+                self.preserved_years,
+                self.in_progress_volumes,
+                self.in_progress_years,
+            )
+        )
 
     @staticmethod
     def name() -> str:
@@ -115,13 +121,22 @@ class LockssPreservation(models.Model):
         :return: A LockssPreservation item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        return utils.preservation_status(LockssPreservation, container_title,
-                                         issn, volume, no=no)
+        return utils.preservation_status(
+            LockssPreservation, container_title, issn, volume, no=no
+        )
 
     @staticmethod
-    def create_preservation(issn, eissn, title, preserved_volumes,
-                            preserved_years, in_progress_volumes,
-                            in_progress_years, publisher, model) -> None:
+    def create_preservation(
+        issn,
+        eissn,
+        title,
+        preserved_volumes,
+        preserved_years,
+        in_progress_volumes,
+        in_progress_years,
+        publisher,
+        model,
+    ) -> None:
         """
         Create a preservation item of this model
         :param issn: the ISSN
@@ -136,12 +151,14 @@ class LockssPreservation(models.Model):
         :return: None
         """
         model.objects.create(
-            issn=issn, eissn=eissn, title=title,
+            issn=issn,
+            eissn=eissn,
+            title=title,
             preserved_volumes=preserved_volumes,
             preserved_years=preserved_years,
             in_progress_volumes=in_progress_volumes,
             in_progress_years=in_progress_years,
-            publisher=publisher
+            publisher=publisher,
         )
 
     @staticmethod
@@ -153,8 +170,9 @@ class LockssPreservation(models.Model):
         :param local: whether the data file is local
         :return: None
         """
-        utils.generic_lockss_import(url, LockssPreservation, local=local,
-                                    skip_first_line=True)
+        utils.generic_lockss_import(
+            url, LockssPreservation, local=local, skip_first_line=True
+        )
 
 
 class CarinianaPreservation(models.Model):
@@ -162,9 +180,9 @@ class CarinianaPreservation(models.Model):
         db_table = "preservationData_carinianapreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['eissn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["eissn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -174,18 +192,23 @@ class CarinianaPreservation(models.Model):
     preserved_years = models.TextField()
     in_progress_volumes = models.TextField()
     in_progress_years = models.TextField()
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (eissn: {}) (v{}({})) ' \
-               '(in progress: {} ({})) '.format(self.title, self.issn,
-                                                self.eissn,
-                                                self.preserved_volumes,
-                                                self.preserved_years,
-                                                self.in_progress_volumes,
-                                                self.in_progress_years
-                                                )
+        return (
+            "{} (issn: {}) (eissn: {}) (v{}({})) "
+            "(in progress: {} ({})) ".format(
+                self.title,
+                self.issn,
+                self.eissn,
+                self.preserved_volumes,
+                self.preserved_years,
+                self.in_progress_volumes,
+                self.in_progress_years,
+            )
+        )
 
     @staticmethod
     def name() -> str:
@@ -202,13 +225,22 @@ class CarinianaPreservation(models.Model):
         :return: A Cariniana item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        return utils.preservation_status(CarinianaPreservation, container_title,
-                                         issn, volume, no=no)
+        return utils.preservation_status(
+            CarinianaPreservation, container_title, issn, volume, no=no
+        )
 
     @staticmethod
-    def create_preservation(issn, eissn, title, preserved_volumes,
-                            preserved_years, in_progress_volumes,
-                            in_progress_years, publisher, model) -> None:
+    def create_preservation(
+        issn,
+        eissn,
+        title,
+        preserved_volumes,
+        preserved_years,
+        in_progress_volumes,
+        in_progress_years,
+        publisher,
+        model,
+    ) -> None:
         """
         Create a preservation item of this model
         :param issn: the ISSN
@@ -223,12 +255,14 @@ class CarinianaPreservation(models.Model):
         :return: None
         """
         model.objects.create(
-            issn=issn, eissn=eissn, title=title,
+            issn=issn,
+            eissn=eissn,
+            title=title,
             preserved_volumes=preserved_volumes,
             preserved_years=preserved_years,
             in_progress_volumes=in_progress_volumes,
             in_progress_years=in_progress_years,
-            publisher=publisher
+            publisher=publisher,
         )
 
     @staticmethod
@@ -240,8 +274,9 @@ class CarinianaPreservation(models.Model):
         :param local: whether the data file is local
         :return: None
         """
-        utils.generic_lockss_import(url, CarinianaPreservation, local=local,
-                                    skip_first_line=True)
+        utils.generic_lockss_import(
+            url, CarinianaPreservation, local=local, skip_first_line=True
+        )
 
 
 class ClockssPreservation(models.Model):
@@ -249,9 +284,9 @@ class ClockssPreservation(models.Model):
         db_table = "preservationData_clocksspreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['eissn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["eissn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -261,18 +296,23 @@ class ClockssPreservation(models.Model):
     preserved_years = models.TextField()
     in_progress_volumes = models.TextField()
     in_progress_years = models.TextField()
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (eissn: {}) (v{}({})) ' \
-               '(in progress: {} ({})) '.format(self.title, self.issn,
-                                                self.eissn,
-                                                self.preserved_volumes,
-                                                self.preserved_years,
-                                                self.in_progress_volumes,
-                                                self.in_progress_years
-                                                )
+        return (
+            "{} (issn: {}) (eissn: {}) (v{}({})) "
+            "(in progress: {} ({})) ".format(
+                self.title,
+                self.issn,
+                self.eissn,
+                self.preserved_volumes,
+                self.preserved_years,
+                self.in_progress_volumes,
+                self.in_progress_years,
+            )
+        )
 
     @staticmethod
     def name() -> str:
@@ -289,13 +329,22 @@ class ClockssPreservation(models.Model):
         :return: A ClockssPreservation item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        return utils.preservation_status(ClockssPreservation, container_title,
-                                         issn, volume, no=no)
+        return utils.preservation_status(
+            ClockssPreservation, container_title, issn, volume, no=no
+        )
 
     @staticmethod
-    def create_preservation(issn, eissn, title, preserved_volumes,
-                            preserved_years, in_progress_volumes,
-                            in_progress_years, publisher, model) -> None:
+    def create_preservation(
+        issn,
+        eissn,
+        title,
+        preserved_volumes,
+        preserved_years,
+        in_progress_volumes,
+        in_progress_years,
+        publisher,
+        model,
+    ) -> None:
         """
         Create a preservation item of this model
         :param issn: the ISSN
@@ -310,12 +359,14 @@ class ClockssPreservation(models.Model):
         :return: None
         """
         model.objects.create(
-            issn=issn, eissn=eissn, title=title,
+            issn=issn,
+            eissn=eissn,
+            title=title,
             preserved_volumes=preserved_volumes,
             preserved_years=preserved_years,
             in_progress_volumes=in_progress_volumes,
             in_progress_years=in_progress_years,
-            publisher=publisher
+            publisher=publisher,
         )
 
     @staticmethod
@@ -327,8 +378,9 @@ class ClockssPreservation(models.Model):
         :param local: whether the data file is local
         :return: None
         """
-        utils.generic_lockss_import(url, ClockssPreservation, local=local,
-                                    skip_first_line=True)
+        utils.generic_lockss_import(
+            url, ClockssPreservation, local=local, skip_first_line=True
+        )
 
 
 class PKPPreservation(models.Model):
@@ -336,22 +388,25 @@ class PKPPreservation(models.Model):
         db_table = "preservationData_pkppreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
     issn = models.CharField(max_length=20)
     preserved_volumes = models.TextField()
     preserved_no = models.TextField(blank=True, null=True)
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (v{}({})) '.format(self.title, self.issn,
-                                                 self.preserved_volumes,
-                                                 self.preserved_no,
-                                                 )
+        return "{} (issn: {}) (v{}({})) ".format(
+            self.title,
+            self.issn,
+            self.preserved_volumes,
+            self.preserved_no,
+        )
 
     @staticmethod
     def name() -> str:
@@ -368,13 +423,14 @@ class PKPPreservation(models.Model):
         :return: A PKPPreservation item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        preserved_item = utils.get_preserved_item_record(PKPPreservation,
-                                                         container_title, issn)
+        preserved_item = utils.get_preserved_item_record(
+            PKPPreservation, container_title, issn
+        )
 
         if not preserved_item or len(preserved_item) == 0:
             return None, False
 
-        if no is not None and no != '' and no != '0':
+        if no is not None and no != "" and no != "0":
             preserved_item.filter(preserved_no=no)
 
             if len(preserved_item) == 0:
@@ -391,12 +447,14 @@ class PKPPreservation(models.Model):
         :param local: whether the data file is local
         :return: None
         """
-        utils.generic_lockss_import(url, PKPPreservation, local=local,
-                                    skip_first_line=True)
+        utils.generic_lockss_import(
+            url, PKPPreservation, local=local, skip_first_line=True
+        )
 
     @staticmethod
-    def create_preservation(issn, title, preserved_volumes, preserved_no,
-                            publisher, model) -> None:
+    def create_preservation(
+        issn, title, preserved_volumes, preserved_no, publisher, model
+    ) -> None:
         """
         Create a preservation item of this model
         :param issn: the ISSN
@@ -407,8 +465,11 @@ class PKPPreservation(models.Model):
         :return: None
         """
         model.objects.create(
-            issn=issn, title=title, preserved_volumes=preserved_volumes,
-            preserved_no=preserved_no, publisher=publisher
+            issn=issn,
+            title=title,
+            preserved_volumes=preserved_volumes,
+            preserved_no=preserved_no,
+            publisher=publisher,
         )
 
 
@@ -417,8 +478,8 @@ class HathiPreservation(models.Model):
         db_table = "preservationData_hathipreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -426,8 +487,9 @@ class HathiPreservation(models.Model):
     preserved_volumes = models.TextField()
 
     def __str__(self):
-        return '{} (issn: {}) (v{}) '.format(self.title, self.issn,
-                                             self.preserved_volumes)
+        return "{} (issn: {}) (v{}) ".format(
+            self.title, self.issn, self.preserved_volumes
+        )
 
     @staticmethod
     def name() -> str:
@@ -444,8 +506,9 @@ class HathiPreservation(models.Model):
         :return: A HathiPreservation item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        preserved_item = utils.get_preserved_item_record(HathiPreservation,
-                                                         container_title, issn)
+        preserved_item = utils.get_preserved_item_record(
+            HathiPreservation, container_title, issn
+        )
 
         if not preserved_item or len(preserved_item) == 0:
             return None, False
@@ -454,8 +517,7 @@ class HathiPreservation(models.Model):
             volume = str(volume)
 
             for pi in preserved_item:
-                vols = [x.strip() for x in
-                        pi.preserved_volumes.split(';')]
+                vols = [x.strip() for x in pi.preserved_volumes.split(";")]
 
                 if volume in vols:
                     return pi, True
@@ -466,29 +528,37 @@ class HathiPreservation(models.Model):
 
     @staticmethod
     @transaction.atomic
-    def import_data(url, bucket="", s3client=None):
+    def import_data(url, bucket="", s3client=None, local=False):
         # download the data file from S3 bucket
         with tempfile.TemporaryDirectory() as tmp:
-            path = Path(tmp) / 'downloaded.file'
+            if local:
+                path = url
+            else:
+                path = Path(tmp) / "downloaded.file"
 
-            hathi_data = utils.download_remote(False, HathiPreservation,
-                                               url, bucket=bucket,
-                                               s3client=s3client,
-                                               decode=False, file=True,
-                                               filename=str(path))
+                hathi_data = utils.download_remote(
+                    False,
+                    HathiPreservation,
+                    url,
+                    bucket=bucket,
+                    s3client=s3client,
+                    decode=False,
+                    file=True,
+                    filename=str(path),
+                )
 
             # clear out existing data
             utils.clear_out(HathiPreservation)
 
             csv.field_size_limit(sys.maxsize)
 
-            with open(str(path), 'r') as input_file:
-                csv_reader = csv.reader(input_file, delimiter='\t')
+            with open(str(path), "r") as input_file:
+                csv_reader = csv.reader(input_file, delimiter="\t")
 
-                volume_matcher = r'v\.\s?(\d+(?:\-?\d+)?)'
-                no_matcher = r'no\.\s?(\d+(?:\-?\d+)?)'
-                year_matcher = r'\d{4}'
-                issn_matcher = r'[0-9][0-9][0-9][0-9][-][0-9][0-9][0-9][X0-9]'
+                volume_matcher = r"v\.\s?(\d+(?:\-?\d+)?)"
+                no_matcher = r"no\.\s?(\d+(?:\-?\d+)?)"
+                year_matcher = r"\d{4}"
+                issn_matcher = r"[0-9][0-9][0-9][0-9][-][0-9][0-9][0-9][X0-9]"
 
                 for row in csv_reader:
                     try:
@@ -513,7 +583,7 @@ class HathiPreservation(models.Model):
                                 issns.append(match)
 
                         # if it's a serial, try to parse the vols
-                        if bf == 'SE' and len(issn) > 0 and vols:
+                        if bf == "SE" and len(issns) > 0 and vols:
                             matches = re.findall(volume_matcher, vols)
 
                             if matches:
@@ -526,7 +596,6 @@ class HathiPreservation(models.Model):
                                     for match in matches:
                                         vols = utils.unpack_range(match)
                                 else:
-
                                     matches = re.findall(year_matcher, vols)
 
                                     if matches:
@@ -536,18 +605,20 @@ class HathiPreservation(models.Model):
                                         unknown_format = True
 
                             if not unknown_format:
-                                if title.endswith('.'):
+                                if title.endswith("."):
                                     title = title[:-1]
 
-                                for issn_no in issn:
+                                for issn_no in issns:
                                     HathiPreservation.objects.create(
                                         issn=issn_no,
                                         title=title,
-                                        preserved_volumes=vols
+                                        preserved_volumes=vols,
                                     )
 
-                                logging.info(f'Added {title} to '
-                                             f'{HathiPreservation.name()} data')
+                                logging.info(
+                                    f"Added {title} (issns {issns}) to "
+                                    f"{HathiPreservation.name()} data"
+                                )
                     except IndexError:
                         pass
 
@@ -557,9 +628,9 @@ class PorticoPreservation(models.Model):
         db_table = "preservationData_porticopreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['eissn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["eissn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -569,17 +640,18 @@ class PorticoPreservation(models.Model):
 
     # this indicates whether the title is preserved or queued
     preserved = models.BooleanField()
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (eissn: {}) (v{}) '.format(self.title,
-                                                         self.issn, self.eissn,
-                                                         self.preserved_volumes)
+        return "{} (issn: {}) (eissn: {}) (v{}) ".format(
+            self.title, self.issn, self.eissn, self.preserved_volumes
+        )
 
     @staticmethod
     def name():
-        return 'Portico'
+        return "Portico"
 
     @staticmethod
     def preservation(container_title, issn, volume, no=None):
@@ -592,15 +664,16 @@ class PorticoPreservation(models.Model):
         :return: A PorticoPreservation item (or None) and a bool indicating
         whether the item is fully preserved
         """
-        preserved_item = utils.get_preserved_item_record(PorticoPreservation,
-                                                         container_title, issn)
+        preserved_item = utils.get_preserved_item_record(
+            PorticoPreservation, container_title, issn
+        )
 
         if not preserved_item or len(preserved_item) == 0:
             return None, False
 
         # Portico gives volume formats as follows:
         # 2013/2014 - v. 2 (1-2)
-        volume_regex = r'v\.\s(\d+)'
+        volume_regex = r"v\.\s(\d+)"
 
         for pi in preserved_item:
             matches = re.findall(volume_regex, pi.preserved_volumes)
@@ -631,24 +704,28 @@ class PorticoPreservation(models.Model):
         csv.field_size_limit(sys.maxsize)
 
         with StringIO(csv_file) as input_file:
-            csv_reader = csv.DictReader(input_file, delimiter='\t',
-                                        quoting=csv.QUOTE_NONE)
+            csv_reader = csv.DictReader(
+                input_file, delimiter="\t", quoting=csv.QUOTE_NONE
+            )
 
             for row in csv_reader:
-                publisher, created = \
-                    Publisher.objects.get_or_create(name=row['publisher_name'])
-
-                PorticoPreservation.objects.create(
-                    issn=row['print_identifier'],
-                    eissn=row['online_identifier'],
-                    title=row['publication_title'],
-                    preserved_volumes=row['holding_list'],
-                    preserved=(row['notes'] == 'Preserved'),
-                    publisher=publisher
+                publisher, created = Publisher.objects.get_or_create(
+                    name=row["publisher_name"]
                 )
 
-                logging.info(f'Added {row["publication_title"]} to '
-                             f'{PorticoPreservation.name()} data')
+                PorticoPreservation.objects.create(
+                    issn=row["print_identifier"],
+                    eissn=row["online_identifier"],
+                    title=row["publication_title"],
+                    preserved_volumes=row["holding_list"],
+                    preserved=(row["notes"] == "Preserved"),
+                    publisher=publisher,
+                )
+
+                logging.info(
+                    f'Added {row["publication_title"]} to '
+                    f"{PorticoPreservation.name()} data"
+                )
 
 
 class OculScholarsPortalPreservation(models.Model):
@@ -656,8 +733,8 @@ class OculScholarsPortalPreservation(models.Model):
         db_table = "preservationData_oculpreservation"
 
         indexes = [
-            models.Index(fields=['issn']),
-            models.Index(fields=['title']),
+            models.Index(fields=["issn"]),
+            models.Index(fields=["title"]),
         ]
 
     title = models.TextField()
@@ -700,17 +777,18 @@ class OculScholarsPortalPreservation(models.Model):
     """
     verified = models.IntegerField()
 
-    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE,
-                                  default=None)
+    publisher = models.ForeignKey(
+        Publisher, on_delete=models.CASCADE, default=None
+    )
 
     def __str__(self):
-        return '{} (issn: {}) (v{}({})) '.format(self.title, self.issn,
-                                                 self.preserved_volumes,
-                                                 self.preserved_no)
+        return "{} (issn: {}) (v{}({})) ".format(
+            self.title, self.issn, self.preserved_volumes, self.preserved_no
+        )
 
     @staticmethod
     def name():
-        return 'OCUL Scholars Portal'
+        return "OCUL Scholars Portal"
 
     @staticmethod
     def preservation(container_title, issn, volume, no=None):
@@ -724,7 +802,8 @@ class OculScholarsPortalPreservation(models.Model):
         indicating whether the item is fully preserved
         """
         preserved_item = utils.get_preserved_item_record(
-            OculScholarsPortalPreservation, container_title, issn)
+            OculScholarsPortalPreservation, container_title, issn
+        )
 
         if not preserved_item or len(preserved_item) == 0:
             return None, False
@@ -741,36 +820,39 @@ class OculScholarsPortalPreservation(models.Model):
             # v24(1)
             # 1, 2, 3-4, 3.4
             # 1 to 4
-            basic_volume_regex = r'v(\d+)'
-            unpack_volume_regex = r'(\d+\-\d+)'
-            unpack_volume_words_regex = r'(\d+ to \d+)'
+            basic_volume_regex = r"v(\d+)"
+            unpack_volume_regex = r"(\d+\-\d+)"
+            unpack_volume_words_regex = r"(\d+ to \d+)"
 
             pis = preserved_item.filter(
-                preserved_volumes__iregex=basic_volume_regex)
+                preserved_volumes__iregex=basic_volume_regex
+            )
 
             if len(pis) > 0:
                 for pi in pis:
-                    matches = re.findall(basic_volume_regex,
-                                         pi.preserved_volumes)
+                    matches = re.findall(
+                        basic_volume_regex, pi.preserved_volumes
+                    )
 
                     if volume in matches:
                         return pi, pi.preserved
 
             pis = preserved_item.filter(
-                preserved_volumes__iregex=unpack_volume_regex)
+                preserved_volumes__iregex=unpack_volume_regex
+            )
 
             if len(pis) > 0:
                 for pi in pis:
-                    matches = re.findall(unpack_volume_regex,
-                                         pi.preserved_volumes)
+                    matches = re.findall(
+                        unpack_volume_regex, pi.preserved_volumes
+                    )
 
                     unpacked_range = utils.unpack_range(matches)
-                    print(f'Range unpacked to: {unpacked_range}')
+                    print(f"Range unpacked to: {unpacked_range}")
 
                     if volume in unpacked_range:
                         return pi, pi.preserved
 
-
             pis = preserved_item.filter(preserved_volumes=volume)
             return None, False
 
@@ -778,7 +860,7 @@ class OculScholarsPortalPreservation(models.Model):
         # v24(1)
         # 1, 2, 3-4, 3.4
         # 1 to 4
-        volume_regex = r'v(\d+)'
+        volume_regex = r"v(\d+)"
 
         for pis in preserved_item:
             print(pis.preserved_volumes)
@@ -795,14 +877,18 @@ class OculScholarsPortalPreservation(models.Model):
     def import_data(url, bucket="", s3client=None):
         # download the data file from S3 bucket
         with tempfile.TemporaryDirectory() as tmp:
-            path = Path(tmp) / 'downloaded.file'
+            path = Path(tmp) / "downloaded.file"
 
-            xml_file = utils.download_remote(False,
-                                             OculScholarsPortalPreservation,
-                                             url, bucket=bucket,
-                                             s3client=s3client,
-                                             decode=False, file=True,
-                                             filename=str(path))
+            xml_file = utils.download_remote(
+                False,
+                OculScholarsPortalPreservation,
+                url,
+                bucket=bucket,
+                s3client=s3client,
+                decode=False,
+                file=True,
+                filename=str(path),
+            )
 
             # local file for testing
             # xml_file = '/home/martin/scholars_portal_keepers_20230202.xml'
@@ -810,13 +896,19 @@ class OculScholarsPortalPreservation(models.Model):
             # clear out
             utils.clear_out(OculScholarsPortalPreservation)
 
-            fields = ['oph:ResourceVersionIDType', 'oph:IDValue',
-                      'oph:FixedCoverage', 'oph:PreservationStatusCode',
-                      'oph:VerificationStatus']
+            fields = [
+                "oph:ResourceVersionIDType",
+                "oph:IDValue",
+                "oph:FixedCoverage",
+                "oph:PreservationStatusCode",
+                "oph:VerificationStatus",
+            ]
 
             xml_parsed = utils.process_onix(
-                xml_file, fields,
-                callback=OculScholarsPortalPreservation.create_preservation)
+                xml_file,
+                fields,
+                callback=OculScholarsPortalPreservation.create_preservation,
+            )
 
     @staticmethod
     def create_preservation(output) -> None:
@@ -826,43 +918,68 @@ class OculScholarsPortalPreservation(models.Model):
         :return: None
         """
 
-        if 'status' not in output or output['status'] is None:
+        if "status" not in output or output["status"] is None:
             status = 0
         else:
-            status = output['status']
+            status = output["status"]
 
-        if 'verified' not in output or output['verified'] is None:
+        if "verified" not in output or output["verified"] is None:
             verified = 0
         else:
-            verified = output['verified']
+            verified = output["verified"]
 
-        if 'publisher' not in output or output['publisher'] is None:
-            publisher = 'Default'
+        if "publisher" not in output or output["publisher"] is None:
+            publisher = "Default"
         else:
-            publisher = output['publisher']
+            publisher = output["publisher"]
 
-        if 'title' not in output or output['title'] is None:
-            title = 'Unknown title'
+        if "title" not in output or output["title"] is None:
+            title = "Unknown title"
         else:
-            title = output['title']
+            title = output["title"]
 
-        publisher, created = \
-            Publisher.objects.get_or_create(name=publisher)
+        publisher, created = Publisher.objects.get_or_create(name=publisher)
 
-        volumes = output['volumes']
-        issues = output['issues']
+        volumes = output["volumes"]
+        issues = output["issues"]
 
         for volume, issue in zip(volumes, issues):
-
-            volume = volume if volume is not None else ''
-            issue = issue if issue is not None else ''
+            volume = volume if volume is not None else ""
+            issue = issue if issue is not None else ""
 
             OculScholarsPortalPreservation.objects.create(
-                issn=output['issn'], title=title,
-                preserved_volumes=volume, preserved_no=issue,
-                preserved=status, verified=verified,
-                publisher=publisher
+                issn=output["issn"],
+                title=title,
+                preserved_volumes=volume,
+                preserved_no=issue,
+                preserved=status,
+                verified=verified,
+                publisher=publisher,
             )
 
-        logging.info(f'Added {title} to '
-                     f'{OculScholarsPortalPreservation.name()} data')
+        logging.info(
+            f"Added {title} to " f"{OculScholarsPortalPreservation.name()} data"
+        )
+
+
+class ISSNL(models.Model):
+    class Meta:
+        db_table = "preservationData_issnl"
+
+        indexes = [
+            models.Index(fields=["identifier"]),
+        ]
+
+    identifier = models.CharField(max_length=255)
+
+
+class ISSN(models.Model):
+    class Meta:
+        db_table = "preservationData_issn"
+
+        indexes = [
+            models.Index(fields=["identifier"]),
+        ]
+
+    identifier = models.CharField(max_length=255)
+    issnl = models.ForeignKey(ISSNL, on_delete=models.CASCADE, default=None)
