@@ -1,6 +1,8 @@
+import csv
 import gc
 import logging
 import re
+import sys
 import tracemalloc
 from csv import DictReader
 from io import StringIO
@@ -15,9 +17,14 @@ from django.db.models import QuerySet, Count
 from lxml import etree as ET
 
 
-def show_preservation(container_title: str, issn: str, volume: str,
-                      no: str | None, doi: str,
-                      archive: str = None) -> (dict | None, str):
+def show_preservation(
+    container_title: str,
+    issn: str,
+    volume: str,
+    no: str | None,
+    doi: str,
+    archive: str = None,
+) -> (dict | None, str):
     """
     Determine whether an item is preserved
     :param container_title: the journal/container name
@@ -29,6 +36,32 @@ def show_preservation(container_title: str, issn: str, volume: str,
     :return: a dictionary of preservations and a doi
     """
     from preservationdatabase import constants
+    from preservationdatabase.models import ISSN, ISSNL
+
+    # extend ISSNs
+    issn = set(issn)
+
+    # extend the ISSN using the ISSNL lookup table
+    final_issn_list = []
+
+    for issn_item in issn:
+        final_issn_list.append(issn_item)
+        issn_objs = ISSN.objects.filter(identifier=issn_item)
+
+        for issn_obj in issn_objs:
+            issnl_obj = ISSNL.objects.filter(identifier=issn_obj.issnl)
+
+            issn_reversed = ISSN.objects.filter(issnl=issn_obj.issnl)
+
+            for reversed in issn_reversed:
+                final_issn_list.append(reversed.identifier)
+
+    final_issn_list = set(final_issn_list)
+
+    if issn != final_issn_list:
+        logging.info(
+            f"Expanded ISSN list for title from {issn} to {final_issn_list}"
+        )
 
     if archive is None:
         preservation_systems = [*constants.archives.values()]
@@ -38,29 +71,32 @@ def show_preservation(container_title: str, issn: str, volume: str,
     preservations = {}
 
     for system in preservation_systems:
-        preserved, done = system.preservation(container_title, issn, volume, no)
+        preserved, done = system.preservation(
+            container_title, final_issn_list, volume, no
+        )
         preservations[system.name()] = preserved, done
 
     return preservations, doi
 
 
 def unpack_range(s: str) -> list:
-    """ Converts a range of numbers to a full set"""
+    """Converts a range of numbers to a full set"""
     r = []
-    for i in s.split(','):
-        if '-' not in i and ' to ' not in i:
+    for i in s.split(","):
+        if "-" not in i and " to " not in i:
             r.append(int(i))
-        elif ' to ' not in i:
-            l, h = map(int, i.split(' to '))
+        elif " to " in i:
+            l, h = map(int, i.split(" to "))
             r += range(l, h + 1)
         else:
-            l, h = map(int, i.split('-'))
+            l, h = map(int, i.split("-"))
             r += range(l, h + 1)
     return r
 
 
-def show_preservation_for_doi(doi_input: str,
-                              archive: str = None) -> (dict | None, str):
+def show_preservation_for_doi(
+    doi_input: str, archive: str = None
+) -> (dict | None, str):
     """
     Determine whether a DOI is preserved with resolution via the REST API
     :param doi_input: the DOI to look up
@@ -71,32 +107,53 @@ def show_preservation_for_doi(doi_input: str,
     # TODO: tests
     # TODO: type hints
 
-    my_etiquette = Etiquette('Preservation Status', '0.01',
-                             'https://eve.gd', 'meve@crossref.org')
+    my_etiquette = Etiquette(
+        "Preservation Status", "0.01", "https://eve.gd", "meve@crossref.org"
+    )
 
     works = Works(etiquette=my_etiquette)
     doi = works.doi(doi_input)
 
     if not doi:
-        logging.info(f'Unable to resolve DOI: {doi_input}')
+        logging.info(f"Unable to resolve DOI: {doi_input}")
         return {}, doi_input
 
-    container_title = doi['container-title'] if 'container-title' \
-                                                in doi else None
-    issn = doi['ISSN'] if 'ISSN' in doi else None
-    volume = doi['volume'] if 'volume' in doi else None
-    no = doi['issue'] if 'issue' in doi else None
+    container_title = (
+        doi["container-title"] if "container-title" in doi else None
+    )
+    issns = set(doi["ISSN"]) if "ISSN" in doi else None
 
-    logging.info(f'Checking {doi["DOI"]}: {container_title} '
-                 f'({issn}) v{volume} n{no}')
+    volume = doi["volume"] if "volume" in doi else None
+    no = doi["issue"] if "issue" in doi else None
+
+    logging.info(
+        f'Checking {doi["DOI"]}: {container_title} '
+        f"({issns}) v{volume} n{no}"
+    )
+
+    preserves, doi_echo = show_preservation(
+        container_title, issns, volume, no, doi, archive
+    )
 
     # the "archive" field is apparently returned if deposited
-    if 'archive' in doi:
-        logging.info(f'{doi_input} asserts preservation in {doi["archive"]}.')
-    else:
-        logging.info(f'{doi_input} asserts no preservation information.')
 
-    return show_preservation(container_title, issn, volume, no, doi, archive)
+    archives = set(doi["archive"]) if "archive" in doi else set()
+
+    if "archive" in doi:
+        for archive in archives:
+            if archive in preserves and preserves[archive][0]:
+                logging.info(
+                    f"{doi_input} correctly asserts preservation in {archive}."
+                )
+            else:
+                logging.info(
+                    f"{doi_input} incorrectly asserts preservation in {archive}."
+                )
+
+    else:
+        logging.info(f"{doi_input} asserts no preservation information.")
+
+    return preserves, doi_input
 
 
 def normalize_doi(doi: str) -> str:
@@ -110,16 +167,16 @@ def normalize_doi(doi: str) -> str:
     # note that this is not as rigorous as it could be, but writing a single
     # expression that captures everything is hard.
     # See: https://www.crossref.org/blog/dois-and-matching-regular-expressions/
-    pattern = r'(10.\d{4,9}/[-._;()/:A-Z0-9]+)'
+    pattern = r"(10.\d{4,9}/[-._;()/:A-Z0-9]+)"
 
     result = re.search(pattern, doi, re.IGNORECASE)
 
     return result.group(0) if result else None
 
 
-def generic_lockss_import(url: str, model,
-                          skip_first_line: bool = False,
-                          local: bool = False) -> None:
+def generic_lockss_import(
+    url: str, model, skip_first_line: bool = False, local: bool = False
+) -> None:
     """
     The generic import function for LOCKSS-like models
     :param url: the URL to download
@@ -146,71 +203,94 @@ def generic_lockss_import(url: str, model,
 
         for row in csv_reader:
             # trim the publisher field if needed
-            publisher_name = row['Publisher'][:254]
+            publisher_name = row["Publisher"][:254]
 
-            publisher, created = \
-                Publisher.objects.get_or_create(name=publisher_name)
+            publisher, created = Publisher.objects.get_or_create(
+                name=publisher_name
+            )
 
-            if model.name() == 'CLOCKSS' or model.name() == 'LOCKSS'\
-                    or model.name() == 'Cariniana':
+            if (
+                model.name() == "CLOCKSS"
+                or model.name() == "LOCKSS"
+                or model.name() == "Cariniana"
+            ):
                 # create the item
                 model.create_preservation(
-                    issn=row['ISSN'], eissn=row['eISSN'], title=row['Title'],
-                    preserved_volumes=row['Preserved Volumes'],
-                    preserved_years=row['Preserved Years'],
-                    in_progress_volumes=row['In Progress Volumes'],
-                    in_progress_years=row['In Progress Years'],
-                    publisher=publisher, model=model
+                    issn=row["ISSN"],
+                    eissn=row["eISSN"],
+                    title=row["Title"],
+                    preserved_volumes=row["Preserved Volumes"],
+                    preserved_years=row["Preserved Years"],
+                    in_progress_volumes=row["In Progress Volumes"],
+                    in_progress_years=row["In Progress Years"],
+                    publisher=publisher,
+                    model=model,
                 )
-            elif model.name() == 'PKP PLN':
+            elif model.name() == "PKP PLN":
                 model.create_preservation(
-                    issn=row['ISSN'], title=row['Title'],
-                    preserved_volumes=row['Vol'],
-                    preserved_no=row['No'],
-                    publisher=publisher, model=model
+                    issn=row["ISSN"],
+                    title=row["Title"],
+                    preserved_volumes=row["Vol"],
+                    preserved_no=row["No"],
+                    publisher=publisher,
+                    model=model,
                 )
 
             logging.info(f'Added {row["Title"]} to {model.name()} data')
 
 
 def clear_out(model):
-    logging.info(f'Clearing previous {model.name()} data')
+    if hasattr(model, "name"):
+        logging.info(f"Clearing previous {model.name()} data")
     model.objects.all().delete()
 
 
-def download_remote(local, model, url, bucket="", s3client=None, decode=True,
-                    file=False, filename=''):
+def download_remote(
+    local,
+    model,
+    url,
+    bucket="",
+    s3client=None,
+    decode=True,
+    file=False,
+    filename="",
+):
     if s3client:
         if not file:
             if decode:
-                return s3client.get_object(Bucket=bucket, Key=url)[
-                    'Body'].read().decode('utf-8')
+                return (
+                    s3client.get_object(Bucket=bucket, Key=url)["Body"]
+                    .read()
+                    .decode("utf-8")
+                )
             else:
                 return s3client.get_object(Bucket=bucket, Key=url)[
-                    'Body'].read()
+                    "Body"
+                ].read()
         else:
-            logging.info('Storing downloaded file as {}'.format(filename))
+            logging.info("Storing downloaded file as {}".format(filename))
             s3client.download_file(bucket, url, filename)
             return filename
 
     if not local:
-        logging.info(f'Downloading: {model.name()} data')
+        logging.info(f"Downloading: {model.name()} data")
         csv_file = requests.get(url).text
     else:
-        logging.info(f'Using local file for {model.name()} data')
+        logging.info(f"Using local file for {model.name()} data")
 
         if decode:
-            with open(url, 'r') as f:
+            with open(url, "r") as f:
                 csv_file = f.read()
         else:
-            with open(url, 'rb') as f:
+            with open(url, "rb") as f:
                 csv_file = f.read()
 
     return csv_file
 
 
-def preservation_status(model, container_title, issn, volume,
-                        no=None) -> (dict | None, str):
+def preservation_status(
+    model, container_title, issn, volume, no=None
+) -> (dict | None, str):
     """
     Determine whether a DOI is preserved in model
     :param model: the model class to use
@@ -227,8 +307,8 @@ def preservation_status(model, container_title, issn, volume,
         return None, False
 
     for pi in preserved_item:
-        vols = [x.strip() for x in pi.preserved_volumes.split(';')]
-        vols_in_prog = [x.strip() for x in pi.in_progress_volumes.split(';')]
+        vols = [x.strip() for x in pi.preserved_volumes.split(";")]
+        vols_in_prog = [x.strip() for x in pi.in_progress_volumes.split(";")]
 
         volume = str(volume)
 
@@ -252,7 +332,7 @@ def get_preserved_item_record(model, container_title, issn) -> QuerySet | None:
 
     # test ISSN
     try:
-        if issn and 'issn' in fields:
+        if issn and "issn" in fields:
             preserved_item = None
 
             for sub_issn in issn:
@@ -267,7 +347,7 @@ def get_preserved_item_record(model, container_title, issn) -> QuerySet | None:
     except model.DoesNotExist:
         # test EISSN
         try:
-            if issn and 'eissn' in fields:
+            if issn and "eissn" in fields:
                 preserved_item = None
 
                 for sub_issn in issn:
@@ -282,7 +362,7 @@ def get_preserved_item_record(model, container_title, issn) -> QuerySet | None:
         except model.DoesNotExist:
             # test container title
             try:
-                if container_title and 'container_title' in fields:
+                if container_title and "container_title" in fields:
                     preserved_item = model.objects.filter(title=container_title)
                     if not preserved_item or len(preserved_item) == 0:
                         raise model.DoesNotExist
@@ -298,21 +378,23 @@ def process_onix(xml_file, elements, callback) -> None:
     """
     A faster method for processing ONIX XML than using DOM methods
     """
-    current_object = {'volumes': [], 'issues': []}
+    current_object = {"volumes": [], "issues": []}
 
     collect = False
 
     count = 0
 
-    for event, elem in ET.iterparse(xml_file, events=('start', 'end')):
-
-        if event == 'end' and elem.tag == '{http://www.editeur.org/onix/' \
-                                          'serials/SOH}HoldingsRecord':
+    for event, elem in ET.iterparse(xml_file, events=("start", "end")):
+        if (
+            event == "end"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}HoldingsRecord"
+        ):
             callback(current_object)
 
             gc.collect()
 
-            current_object = {'volumes': [], 'issues': []}
+            current_object = {"volumes": [], "issues": []}
 
             count = count + 1
             stats = []
@@ -323,46 +405,67 @@ def process_onix(xml_file, elements, callback) -> None:
                 django.db.reset_queries()
 
             # free memory (urgh)
+            # a note: without this, the elementree will simply grow in memory
+            # and previously iterated elements will not be garbage collected
             elem.clear()
 
         # detect if it's an ISSN
-        if event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}ResourceVersionIDType' \
-                and elem.text == '07':
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}ResourceVersionIDType"
+            and elem.text == "07"
+        ):
             collect = True
-        elif event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}ResourceVersionIDType' \
-                and elem.text != '07':
+        elif (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}ResourceVersionIDType"
+            and elem.text != "07"
+        ):
             collect = False
 
-        if event == 'start' and elem.tag == '{http://www.editeur.org/onix/' \
-                                            'serials/SOH}IDValue' and collect:
-            current_object['issn'] = normalize_issn(elem.text)
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}IDValue"
+            and collect
+        ):
+            current_object["issn"] = normalize_issn(elem.text)
 
-        if event == 'start' and elem.tag == '{http://www.editeur.org/onix/' \
-                                            'serials/SOH}TitleText' and collect:
-            current_object['title'] = elem.text
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}TitleText"
+            and collect
+        ):
+            current_object["title"] = elem.text
 
-        if event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}PublisherName' \
-                and collect:
-            current_object['publisher'] = elem.text
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}PublisherName"
+            and collect
+        ):
+            current_object["publisher"] = elem.text
 
-        if event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}Coverage' and collect:
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}Coverage"
+            and collect
+        ):
             coverage = elem
 
-            volume = ''
-            issue = ''
+            volume = ""
+            issue = ""
 
             level_one = coverage.find(
-                ".//{http://www.editeur.org/onix/serials/SOH}Level1")
+                ".//{http://www.editeur.org/onix/serials/SOH}Level1"
+            )
             level_two = coverage.find(
-                ".//{http://www.editeur.org/onix/serials/SOH}Level2")
+                ".//{http://www.editeur.org/onix/serials/SOH}Level2"
+            )
 
             levels = [level_one, level_two]
             unit = None
@@ -370,39 +473,45 @@ def process_onix(xml_file, elements, callback) -> None:
             for level in levels:
                 if level:
                     unit = level.find(
-                        ".//{http://www.editeur.org/onix/serials/SOH}Unit")
+                        ".//{http://www.editeur.org/onix/serials/SOH}Unit"
+                    )
 
                     # no idea why, but this HAS to be an explicit comparison
                     # with None rather than a boolean comparison
                     unit = unit.text if unit is not None else None
 
                     number = level.find(
-                        ".//{http://www.editeur.org/onix/serials/SOH}Number")
+                        ".//{http://www.editeur.org/onix/serials/SOH}Number"
+                    )
 
                     number = number.text if number is not None else None
 
-                    if unit == 'Volume':
+                    if unit == "Volume":
                         volume = number
-                    elif unit == 'Issue':
+                    elif unit == "Issue":
                         issue = number
                     else:
-                        print('Unrecognised unit: {}'.format(unit))
+                        print("Unrecognised unit: {}".format(unit))
 
             if unit:
-                current_object['volumes'].append(volume)
-                current_object['issues'].append(issue)
+                current_object["volumes"].append(volume)
+                current_object["issues"].append(issue)
 
-        if event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}PreservationStatusCode'\
-                and collect:
-            current_object['status'] = elem.text
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}PreservationStatusCode"
+            and collect
+        ):
+            current_object["status"] = elem.text
 
-        if event == 'start' \
-                and elem.tag == '{http://www.editeur.org/onix/' \
-                                'serials/SOH}VerificationStatus'\
-                and collect:
-            current_object['verification'] = elem.text
+        if (
+            event == "start"
+            and elem.tag == "{http://www.editeur.org/onix/"
+            "serials/SOH}VerificationStatus"
+            and collect
+        ):
+            current_object["verification"] = elem.text
 
     return
 
@@ -413,9 +522,9 @@ def normalize_issn(issn) -> str:
     :param issn: the ISSN
     :return: the normalized ISSN
     """
-    issn = '' if issn is None else issn.strip()
+    issn = "" if issn is None else issn.strip()
 
-    return f'{issn[0:4]}-{issn[4:8]}' if issn and '-' not in issn else issn
+    return f"{issn[0:4]}-{issn[4:8]}" if issn and "-" not in issn else issn
 
 
 @transaction.atomic
@@ -450,28 +559,56 @@ def in_archive(item, archive) -> bool:
 
     fields = item._meta.fields
 
-    if 'issn' in fields:
-        issns = archive.objects.filter(issn=item['issn'])
+    if "issn" in fields:
+        issns = archive.objects.filter(issn=item["issn"])
 
         if len(issns) > 0:
             return True
 
-    if 'eissn' in fields:
+    if "eissn" in fields:
         try:
-            issns = archive.objects.filter(eissn=item['eissn'])
+            issns = archive.objects.filter(eissn=item["eissn"])
 
             if len(issns) > 0:
                 return True
         except:
-            issns = archive.objects.filter(issn=item['eissn'])
+            issns = archive.objects.filter(issn=item["eissn"])
 
             if len(issns) > 0:
                 return True
 
-    if 'title' in fields:
-        titles = archive.objects.filter(title=item['title'])
+    if "title" in fields:
+        titles = archive.objects.filter(title=item["title"])
 
         if len(titles) > 0:
             return True
 
     return False
+
+
+def import_issnl(issnl_file, issnl_model, issn_model):
+    from rich.progress import track
+
+    csv.field_size_limit(sys.maxsize)
+
+    clear_out(issn_model)
+    clear_out(issnl_model)
+
+    logging.info("Erased existing ISSN-ISSNL mappings.")
+
+    with open(issnl_file, "r") as input_file:
+        csv_reader = csv.DictReader(input_file, delimiter="\t")
+
+        for row in track(csv_reader):
+            if row["ISSN"] != row["ISSN-L"]:
+                issnl, created = issnl_model.objects.get_or_create(
+                    identifier=row["ISSN-L"]
+                )
+
+                issn, created = issn_model.objects.get_or_create(
+                    identifier=row["ISSN"], issnl=issnl
+                )
+
+                logging.info(
+                    f"Created ISSN-ISSNL mapping: {issn.identifier} {issnl.identifier}"
+                )
